/*
 * bst.c
 *
 *  Created on: Nov 5, 2013
 *      Author: mtk
 */
#include <stdlib.h>
#include <stdio.h>
#include "bst.h"

void init_bst(bst* t){
	t->root = NULL;
}

bst_element minimum(bst* t){
	if( t->root == NULL ){
		fprintf(stderr,"Error: empty BST.\n");
		return 0;
	}
	bst_node* current = t->root;
	while( current->left != NULL){
		current = current->left;
	}
	return current->key;
}


bst_element maximum(bst* t){
	if( t->root == NULL ){
		fprintf(stderr,"Error: empty BST.\n");
		return 0;
	}
	bst_node* current = t->root;
	while( current->right != NULL){
		current = current->right;
	}
	return current->key;
}

bst_node* search_node(bst* t, bst_element e){
	if( t->root == NULL ){
		return NULL;
	}
	bst_node* current = t->root;
	while( current!=NULL && current->key!=e){
		if( e < current->key ) current = current->left;
		else current = current->right;
	}
	return current;
}

bst_element search(bst* t, bst_element e){
	bst_node* current = search_node(t,e);
	if(current) return current->key;
	else return 0;
}

bst_node* predecessor_node(bst_node* n){
	if( n->left != NULL){
		bst_node* current = n->left;
		while( current->right != NULL){
			current = current->right;
		}
		return current;
	}
	bst_node* p=n->parent;
	while( p!=NULL && n != p->right){
		n=p;
		p=p->parent;
	}
	return p;
}

bst_element predecessor(bst* t, bst_element e){
	bst_node* n = search_node(t,e);
	if(n==NULL){
		fprintf(stderr,"Error: cannot find specified element.\n");
		return 0;
	}
	n = predecessor_node(n);
	if(n==NULL){
		fprintf(stderr,"Error: cannot find predecessor in BST.\n");
		return 0;
	}
	return n->key;
}

bst_node* successor_node(bst_node* n){
	if( n->right != NULL){
		bst_node* current = n->right;
		while( current->left != NULL){
			current = current->left;
		}
		return current;
	}
	bst_node* p=n->parent;
	while( p!=NULL && n != p->left){
		n=p;
		p=p->parent;
	}
	return p;
}
bst_element successor(bst* t, bst_element e){
	bst_node* n = search_node(t,e);
	if(n==NULL){
		fprintf(stderr,"Error: cannot find specified element.\n");
		return 0;
	}
	n = successor_node(n);
	if(n==NULL){
		fprintf(stderr,"Error: cannot find successor in BST.\n");
		return 0;
	}
	return n->key;
}

/* insert operations */
void insert(bst* t, bst_element e){
	bst_node* n = malloc(sizeof(bst_node));
	n->key = e;
	n->parent = NULL;
	n->left = NULL;
	n->right = NULL;

	bst_node* p = NULL;
	bst_node* c = t->root;

	if(c==NULL){
		t->root = n;
		return;
	}

	while(c!=NULL){
		p=c;
		if(e < c->key) c=c->left;
		else c=c->right;
	}
	n->parent = p;
	if( e < p->key) p->left = n;
	else p->right = n;
}

void delete(bst* t, bst_element e){

}

